﻿Shader "Custom/3p_map"
{
	Properties
	{
		_Color ("Color", Color) = (1,1,1,1)
		_MainTex ("Albedo (RGB)", 2D) = "white" {}
		_Glossiness ("Smoothness", Range(0,1)) = 0.5
		_Metallic ("Metallic", Range(0,1)) = 0.0
	}
	SubShader
	{
		Tags { "RenderType"="Opaque" }
		LOD 200

		CGPROGRAM
		// Physically based Standard lighting model, and enable shadows on all light types
		#pragma surface surf Standard fullforwardshadows

		// Use shader model 3.0 target, to get nicer looking lighting
		#pragma target 3.0

		sampler2D _MainTex;

		struct Input
		{
			// float2 uv_MainTex;
			float3 worldPos;
			float3 worldNormal;
		};

		half _Glossiness;
		half _Metallic;
		fixed4 _Color;

		// Add instancing support for this shader. You need to check 'Enable Instancing' on materials that use the shader.
		// See https://docs.unity3d.com/Manual/GPUInstancing.html for more information about instancing.
		// #pragma instancing_options assumeuniformscaling
		UNITY_INSTANCING_BUFFER_START(Props)
			// put more per-instance properties here
		UNITY_INSTANCING_BUFFER_END(Props)

		void surf (Input IN, inout SurfaceOutputStandard o)
		{
			// Albedo comes from a texture tinted by color
			// fixed4 c = tex2D (_MainTex, IN.uv_MainTex) * _Color;
			// float3 amounts = IN.worldNormal / (IN.worldNormal.x+IN.worldNormal.y+IN.worldNormal.z);
			float3 amounts = float3(
				dot(IN.worldNormal, float3(1,0,0)),
				dot(IN.worldNormal, float3(0,1,0)),
				dot(IN.worldNormal, float3(0,0,1)));
			amounts = abs(amounts);
			float3 uv = frac(IN.worldPos);
			fixed3 c = _Color
			 *(
				 tex2D (_MainTex, uv.xy) /*float3(0,0,1)*/ * amounts.z
			 +  tex2D (_MainTex, uv.xz) /*float3(0,1,0)*/ * amounts.y
			 +  tex2D (_MainTex, uv.yz) /*float3(1,0,0)*/ * amounts.x);
			o.Albedo = c.rgb;
			// Metallic and smoothness come from slider variables
			o.Metallic = _Metallic;
			o.Smoothness = _Glossiness;
			// o.Alpha = c.a;
		}
		ENDCG
	}
	FallBack "Diffuse"
}
